package rocketBugsRender;
/*
	PairWiseDisp... class that analyzes for steps
*/

/*  <RocketBugsRender - for rendering and graphing the output of Zoom (and other programs)>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.*;
import java.awt.event.*;
import java.lang.Math;
import javax.swing.*;
import javax.swing.event.*;
import java.io.*;
import java.text.*;

public class PairWiseDisp extends JFrame implements ActionListener, WindowListener, ComponentListener, KeyListener, MouseListener, MouseMotionListener, AdjustmentListener {
	DataSet inSet;
	DataColumn pathCol;
	DataColumn timeCol;	
	Dimension frameDim = new Dimension(400,400);
 	JFileChooser opener = new JFileChooser();
 	JProgressBar pairsProgress = new JProgressBar(0,1);
 	Timer progressTimer = new Timer (1000, this);
 	File pairWiseFile;
 	String pairWisePath;
 	boolean working = false;
 	
 	// analysis params DEFAULT
	static final int NumberOfPasses = 10000;	// number of intervals to use in making histogram
	static final double TimeInterval = 0.1;	// time interval in which to calculate pairwise distances
	static final double StartTime = 2.0;	// start time of analysis
	static final double StopTime = 1e6;	// a big number, default for whole run
	
	// analysis params
	int numPasses;	// number of intervals to use in making histogram
	double tInterval, startTime, stopTime, deltaT;
	int start,stop;			// indices for start and stop of analysis
	int curPass = 0;
	
	// histogram properties
	int ptCt = 0;
	double binRange = 0.01;
	int numBins = 250;
	double binSize = binRange/numBins;
	int [] hist = new int [numBins];
	double [] bins = new double [numBins];
	
	// panel components
 	JPanel funcPanel = new JPanel();
 	JPanel paramPanel = new JPanel();
	
	public PairWiseDisp () {
		setTitle (" Pairwise Displacements ");
		setSize(frameDim);
		setResizable(true);
		setLocation(50,0);
		
		// make panels
		makeFuncPanel();
		makeParamPanel();
		
		addKeyListener(this);
		addComponentListener(this);
		addMouseListener(this);
		addMouseMotionListener(this);
		addWindowListener(this);
		
		initialize();
	}
	
	class PairsCalc extends Thread {
		
		PairsCalc () {
			this.start();
		}
		
		public void run() {
			pairWiseDisps();
		}
	}
	

	
	public void initialize () {
		show();
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add (funcPanel, BorderLayout.NORTH);
		getContentPane().add (paramPanel, BorderLayout.SOUTH);
		show();
		requestFocus();
	}
	
	public void setParamsToDefault() {
		tInterval = TimeInterval;
		startTime = StartTime;
		stopTime = StopTime;
		numPasses = NumberOfPasses;
	}
	
	public void newPairWise () {
		setParamsToDefault();
		pairWiseFile = null;
		loadPairWiseSet();
		if (pairWiseFile != null) {
			//scroll bar params
			pairsProgress.setMinimum(0);
			pairsProgress.setMaximum(numPasses);
			progressTimer.start();
			// ****************
			
			new PairsCalc();
			
		}
	}
	
	public void loadPairWiseSet() {
		opener.setFileSelectionMode(JFileChooser.FILES_ONLY);
		int fileQuery = opener.showOpenDialog(this);
		if (fileQuery == JFileChooser.APPROVE_OPTION) {
            pairWiseFile = opener.getSelectedFile();
            pairWisePath = pairWiseFile.getAbsolutePath();
            inSet = new DataSet (pairWisePath);
        } else {
            talkln ("Open command cancelled by user");
            return;
        }
	}
	
	public void pairWiseDisps () {
		working = true;
		timeCol = getTimeCol();
		pathCol = getPathCol();
		setAnalysisIndices();
		HistogramPlus histo = new HistogramPlus(250, 0, 0.1, pairWisePath, "pairWise", this);
		talkln ("Time Column = " + String.valueOf(timeCol.name) + " , Path Column = " + String.valueOf(pathCol.name));
		talkln ("Start Time = " + String.valueOf(startTime) + " , Stop Time = " + String.valueOf(stopTime));
		talkln ("Start Index = " + String.valueOf(start) + " , Stop Index = " + String.valueOf(stop));
		// loop through data with random starting times
		deltaT = timeCol.data[start+1]-timeCol.data[start];
		int pairWiseSpan = (int)(tInterval/deltaT);
        double curStartTime = 0;
        double curDisp;
        int curStart, curStop;
        curPass = 0;
        while (curPass < numPasses) {
            curStartTime = Math.random()*(stopTime-startTime-tInterval)+startTime;
            curStart = getIndexFromTime(curStartTime);
            curStop = curStart + pairWiseSpan;
            for (int i=curStart;i<curStop;i++) {
                for (int j=i+1;j<curStop;j++) {
                    //talkln ("i,j = " + String.valueOf(i) + "," + String.valueOf(j));
                    curDisp = Math.abs(pathCol.data[i] - pathCol.data[j]);
                    //talkln ("curDisp = " + String.valueOf(curDisp));
                    histo.addValue(curDisp);
                }
            }
            curPass++;
        }
		working = false;
		FileOps.mkHistogramFile(histo);
		
	}
	
	private DataColumn getTimeCol () {
		return inSet.dataTable[0];
	}
	
	private DataColumn getPathCol () {
		return inSet.dataTable[1];
	}
	
	private int getIndexFromTime(double time) {
		int index = -1;
		for (int i=start;i<=stop;i++) {
			if (timeCol.data[i] > time) {
				index = i;
				break;
			}
		}
		return index;
	}
	
	private void setAnalysisIndices() {
		// find start int from start time
		start = 0;
		for (int i=0;i<timeCol.rows;i++) {
			if (timeCol.data[i] > startTime) { 
				start = i;
				startTime = timeCol.data[i];	// set startTime to actual value at i
				break;
			}
		}
		
		// find stop int from stop time
		stop = timeCol.rows-1;
		for (int i=timeCol.rows-1;i>0;i+=-1) {
			if (timeCol.data[i] < stopTime) {
				stop = i;
				stopTime = timeCol.data[i];		// set stopTime to actual value at i
				break;
			}
		}
	}
	
	
	public void actionPerformed (ActionEvent e) {
		String act = e.getActionCommand();
		
		if (working) {
			pairsProgress.setValue(curPass);
		}
		
		if (!working) {
			progressTimer.stop();
			curPass = 0;
			pairsProgress.setValue(curPass);
		}
		
		if (act != null) {
			if (act.equals("New Pairwise")) {
				newPairWise();
			}
		}
	}
	
	public void adjustmentValueChanged (AdjustmentEvent e) {
	}
	
	
	public void windowActivated (WindowEvent e) {}
	public void windowClosing (WindowEvent e) {}
	public void windowDeactivated (WindowEvent e) {}
	public void windowDeiconified (WindowEvent e) {}
	public void windowIconified (WindowEvent e) {}
	public void windowOpened (WindowEvent e) {}
	public void windowClosed (WindowEvent e) {}
	
	public void keyReleased (KeyEvent e) {}
	public void keyTyped (KeyEvent e) {}
	public void keyPressed (KeyEvent e) {
		String act = e.getKeyText(e.getKeyCode());
		
		if (act.equals("P")) {
			newPairWise();
		}
		
	}
	
	public void mouseClicked (MouseEvent e) {}
	public void mouseEntered (MouseEvent e) {}
	public void mouseExited (MouseEvent e) {}
	public void mousePressed (MouseEvent e) {
		int xPos = e.getX();
		int yPos = e.getY();
	}
	
	public void mouseReleased (MouseEvent e) {}
	public void mouseDragged (MouseEvent e) {}
	public void mouseMoved (MouseEvent e) {}
	
	public void componentHidden (ComponentEvent e) {}
	public void componentShown (ComponentEvent e) {}
	public void componentMoved (ComponentEvent e) {}
	public void componentResized (ComponentEvent e) {}
	
	
	
	public void makeFuncPanel() {
		JButton pairsButton = new JButton ("New Pairwise");
		pairsButton.setFont(Env.controlFont);
		pairsButton.setActionCommand("New Pairwise");
		pairsButton.setToolTipText("Begin a new pairwise calculation");
		pairsButton.addActionListener(this);
		pairsButton.setSize(10,10);
		
		funcPanel.setLayout(new GridLayout(1,1));
		funcPanel.add(pairsButton);
		
		
		
	}
	
	public void makeParamPanel() {
		paramPanel.setLayout(new GridLayout(1,1));
		paramPanel.add(pairsProgress);
	}
	
	private void talk (String info) {
		System.out.print(info);
	}
	
	private void talkln (String info) {
		System.out.println(info);
	}
	
	
	
	
	
	
}